package eMdP.colisions;

import java.util.LinkedList;
import java.util.logging.Logger;

import com.jme.intersection.CollisionData;
import com.jme.intersection.CollisionResults;
import com.jme.intersection.PickData;
import com.jme.intersection.PickResults;
import com.jme.intersection.TriangleCollisionResults;
import com.jme.intersection.TrianglePickResults;
import com.jme.math.Ray;
import com.jme.math.Vector3f;
import com.jme.util.Timer;
import com.jmex.physics.PhysicsSpace;

import eMdP.classesBasiques.Bustia;
import eMdP.classesBasiques.Escenari;
import eMdP.classesBasiques.Furgoneta;
import eMdP.classesBasiques.Personatge;
import eMdP.classesBasiques.PersonatgeNJ;
import eMdP.estats.Partida;
import eMdP.gestors.GestorConstants;
import eMdP.gestors.GestorMusica;

/**
 * Controlador de les Col·lisions. Aquesta classe gestiona totes les
 * col·lisions. <br>
 * Hi han 9 tipus de Colisions al eMdP:<br>
 * 
 * 0. Personatge - Terreny -> Rebota normal<br>
 * 1. Personatge - PNJ -> X segons quiet depenen del PNJ<br>
 * 2. Personatge - Paquet -> El Personatge perd 2 paquets i està 2 segons quiet,
 * estabilitzant-se, excepte si està inmune.<br>
 * 3. Personatge - FurgoC -> Carrega la saca de paquets<br>
 * <br>
 * 4. PNJ - Terreny -> Rebota normal<br>
 * 5. PNJ - Paquet -> El PNJ està 2 segons quiet<br>
 * <br>
 * 6. Paquet - Bustia Bona -> Paquet entra i suma punts al Personatge<br>
 * 7. Paquet - Bustia Mala -> Paquet rebota en la direcció del Personatge (si el
 * toca, col·lisió 1)<br>
 * <br>
 * 8. Paquet - Terreny -> El Paquet desapareix amb una animació<br>
 * 
 */
public class DeteccioColisions {

	private static final Logger logger = Logger.getLogger(Partida.class.getName());

	private PhysicsSpace espaiFisic;

	private LinkedList<Personatge> jugadors;
	private LinkedList<Furgoneta> furgonetes;
	private LinkedList<PersonatgeNJ> PNJS;
	private LinkedList<Bustia> busties;
	private Escenari escenari;

	//private Timer timer;

	public DeteccioColisions(LinkedList<Personatge> jugadors,
			LinkedList<PersonatgeNJ> PNJS, LinkedList<Furgoneta> furgonetes,
			LinkedList<Bustia> busties, Escenari escenari, PhysicsSpace espaiFisic) {
		this.jugadors = jugadors;
		this.PNJS = PNJS;
		this.furgonetes = furgonetes;
		this.busties = busties;
		this.escenari = escenari;

		this.espaiFisic = espaiFisic;
		//Timer.setTimer(timer);
		//this.timer = Timer.getTimer();
	}

	/**
	 * 
	 * @param player
	 */
	private void retrocedir(Personatge node) {
		System.out.println("Retrocedir!!!!!!!!!!!!!");

		Vector3f loc = node.getLocalTranslation();
		Vector3f direccio = new Vector3f();

		node.getLocalRotation().getRotationColumn(2, direccio);
		Ray ray = new Ray(loc, direccio);
		PickResults results = new TrianglePickResults();
		results.setCheckDistance(true);
		node.getParent().findPick(ray, results);
		if (results.getNumber() > 0) {
			PickData closest = results.getPickData(0);
			Vector3f modul = new Vector3f();
			closest.getTargetMesh().getLocalTranslation().subtract(direccio,
					modul);

			if (modul.length() > 0) {
				// if(closest.getDistance()<4){ // Ha xocat de cares
				loc.subtractLocal(direccio.multLocal(0.025f));
				System.out.println("Xocat de CAressssss");
			} else { // Ha xocat d'esquenes
				loc.addLocal(direccio.multLocal(0.025f));
				System.out.println("Xocat d'esquenes");
			}
			node.setLocalTranslation(loc);
			System.out.println("Hem retrocedit");
		}

	}

	/**
	 * Nou mètode per calcular les col·lisions, pensat per multiplayer, multi
	 * PNJS i multi busties
	 */
	public void ProcessCollisionsNou() { // Aquí hauriem de programar les 6
											// colisions!
		CollisionResults colisionats = new TriangleCollisionResults();

		/* Comprovem les col·lisions de tots els jugadors */
		for (int i = 0; i < jugadors.size(); i++) {
			/*
			 * Busquem les colisions que te el jugador, amb algun fill de
			 * l'escenari
			 */
			jugadors.get(i).findCollisions(escenari, colisionats);
			/* Si hi ha alguna col·lisió, la processem */
			if (colisionats.getNumber() > 0) {
				/* Fem un recorregut per totes les col·lisions */
				for (int j = 0; j < colisionats.getNumber(); j++) {
					CollisionData colisio = colisionats.getCollisionData(i);
					boolean trobat = false;
					int k = 0;
					/*
					 * Fem una cerca per saber si ha col·lisionat amb un altre
					 * jugador
					 */
					while (!trobat && k < jugadors.size()) {
						trobat = (colisio.getTargetMesh().hasAncestor(jugadors.get(k)) && k!=i);
						if (trobat){
							System.out.println("DeteccioColisions::Col·lisionem amb un Jugador");
							retrocedir(jugadors.get(i));
						}
						k++;
					}
					k = 0;
					/*
					 * Mentres no trobat fem una cerca per saber si ha
					 * col·lisionat amb un PNJ
					 */
					while (!trobat && k < PNJS.size()) {
						trobat = colisio.getTargetMesh().hasAncestor(
								PNJS.get(k));
						if (trobat){
							/* Si trobat, significa que ha col·lisionat amb un PNJ */
							System.out.println("DeteccioColisions::Col·lisionem amb un PNJ");
							logger.info("COL·LISIONS::Jugador" + jugadors.get(i).getIdJugador()+ " ha colisionat amb PNJ. Tipus 1."+ PNJS.get(k).getName());
							/* Comprovem que el Personatge no està bloquejat */
							if (!jugadors.get(i).estaBloquejat(Timer.getTimer().getTimeInSeconds())) {
								/* Si el PNJ està bloquejat no passa res per col·lisionar amb ell */
								if (!PNJS.get(k).estaBloquejat(Timer.getTimer().getTimeInSeconds())) {
									/* Si cap dels dos està bloquejat, blocagem els dos */
									jugadors.get(i).bloquejar(Timer.getTimer().getTimeInSeconds() + 8);
									PNJS.get(k).bloquejar(Timer.getTimer().getTimeInSeconds() + 14);
								}
							}
							retrocedir(jugadors.get(i));
						}
						k++;
					}
					k = 0;
					/*
					 * Mentres no trobat fem una cerca per saber si ha
					 * col·lisionat amb una bustia
					 */
					/*while (!trobat && k < busties.size()) {
						trobat = colisio.getTargetMesh().hasAncestor(
								busties.get(k));
						if (trobat){
							// Si trobat, significa que ha col·lisionat amb una Bustia 
							System.out.println("DeteccioColisions::Col·lisionem amb una Bustia");
							// Si col·lisionem amb una bústia fem com si col·lisionesim amb l'escenari normal 
							retrocedir(jugadors.get(i));
						}
						k++;
					}
					k = 0;*/
					/*
					 * Mentres no trobat fem una cerca per saber si ha
					 * col·lisionat amb una furgoneta
					 */
					while (!trobat && k < furgonetes.size()) {
						trobat = colisio.getTargetMesh().hasAncestor(furgonetes.get(k));
						if (trobat){
							/* Si trobat, significa que ha col·lisionat amb una Furgoneta */
							System.out.println("DeteccioColisions::Col·lisionem amb una Furgoneta. Tipus 3.");
							logger.info("COL·LISIONS::Jugador col·lisiona amb la Furgoneta. Tipus 3");
							retrocedir(jugadors.get(i));
							
							/* Comprovem que el jugador no tingui paquets, i que la furgoneta hagi arribat al punt de descarregar */
							/*if (jugadors.get(i).quantsFills() < 2 && furgonetes.get(k).potDescarregar()) { 
								/* Comprovem que el jugador no estigui bolquejat */
								/*if (!jugadors.get(i).estaBloquejat(Timer.getTimer().getTimeInSeconds())) {
									/* Carreguem:: Reproduim musica de col·lisio, bloquegem el jugador i el posem immune, i carreguem 4 Paquets */ 
									/*GestorMusica.reproduirSoColisio(0, furgonetes.get(k).getLocalTranslation());
									jugadors.get(i).setImmune(Timer.getTimer().getTimeInSeconds() + 6); 
									jugadors.get(i).bloquejar(Timer.getTimer().getTimeInSeconds() + 3.5f);
									/* Carreguem els 4 paquets */
								/*	int ultimColor = -1;
									for (int l = 0; l < GestorConstants.NUM_PAQUETS_CARREGAMENT; l++)
									{
										int color = (int)Math.round((Math.random()*GestorConstants.NUM_COLORS)); //Creem un num aleatori per tal de escollir el color del paquet
										if(ultimColor != -1)
										{
											if(color == ultimColor)
												l--;
											else
											{
												jugadors.get(i).assignarPaquet(espaiFisic, color);
												ultimColor = color;
											}
												
										}
										else
										{
											jugadors.get(i).assignarPaquet(espaiFisic, color);
											ultimColor = color;
										}
											
										
									}
										
									/* Indiquem a la furgoneta que ha fet una descàrrega */
								/*	furgonetes.get(k).unCarregamentFet(Timer.getTimer().getTimeInSeconds());
									/* Si la furgoneta està buida, carreguem la següent */
									/*if (furgonetes.get(k).getCarregaments() < 1){
										/* Si es la ultima furgoneta de la llista carreguem la primera */
										/*if(k+1 < furgonetes.size())
											furgonetes.get(k+1).setCarregaments(1);
										else
											furgonetes.get(0).setCarregaments(1);
									}
								}
							}
						}
						k++;
					}

					/* Si no hem trobat significa que hem col·lisionat amb l'escenari */
					//if (!trobat) {
						//System.out.println("DeteccioColisions::Col·lisionem amb l'escenari");
						//retrocedir(jugadors.get(i));
				//	}
				//}
			} else {
				//System.out.println("No em colisionat amb ning");
			}

		}
	}

}}}}
